<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Filename:    we_baseElement.inc.php
 *
 * Function:    Utility class that implements basic html elements operations
 *
 * Description: Provides functions for creating html tags
 */
class we_html_baseElement{
	const SELF_CLOSE = 'selfclose';

	protected $tag_name = '';
	protected $need_end_tag = true;

	/** @var array<string,array|string> */
	protected $attribs = ['style' => []];
	protected $content = '';

	/**
	 * Constructor
	 * @param		$attribs								array
	 * @param		$content								string
	 *
	 */
	public function __construct(string $tagname, /* mixed */ $need_end_tag = true, array $attribs = [], string $content = ''){
		$this->tag_name = $tagname;
		$this->need_end_tag = $need_end_tag;
		$this->setAttributes($attribs);
		$this->content = $content;
	}

	/**
	 * Function returns copy of object
	 *
	 * @return     we_html_baseElement
	 */
	protected function copy(): we_html_baseElement{
		return clone $this;
	}

	/* Function sets element attributes
	 * @param		array<string,string|string[]> $attribs
	 * @return		void
	 */

	public function setAttributes(array $attribs): void{
		if(empty($attribs)){
			return;
		}
		foreach($attribs as $k => $v){
			'@phan-var string $k';
			if($k){
				switch($k){
					case 'style':
						$this->setStyleAttrib(is_array($v) ? $v : explode(';', rtrim($v, '; ')));
						break;
					default:
						'@phan-var string $v';
						$this->setAttribute($k, (string) $v);
				}
			}
		}
	}

	/**
	 *
	 * @param string[] $attrib_values
	 * @return void
	 */
	public function setStyleAttrib(array $attrib_values): void{
		foreach($attrib_values as $val){
			$val = trim((string) $val);
			if($val){
				[$k, $v] = explode(':', $val);
				$this->setStyle(trim($k), trim($v));
			}
		}
	}

	/**
	 * Function sets element attribute
	 *
	 * @param		$attrib_name							string
	 * @param		$attrib_value							string
	 *
	 * @return		void
	 */
	public function setAttribute(string $attrib_name, string $attrib_value): void{
		switch($attrib_name){
			case 'style':
				throw new we_exception_base('unsupported call');
			case 'valign':
				$this->setStyle('vertical-align', $attrib_value);
				break;
			case 'height':
			case 'width':
				$this->setStyle($attrib_name, $attrib_value . (is_numeric($attrib_value) ? 'px' : ''));
				break;
			case 'border':
				$this->setStyle('border-width', $attrib_value . (is_numeric($attrib_value) ? 'px' : ''));
				break;
			case 'bgcolor':
				$this->setStyle('background-color', $attrib_value);
				break;
			case 'background':
				$this->setStyle('background-image', 'url(' . $attrib_value . ')');
				break;
			case 'disabled':
			case 'multiple':
			case 'noshade':
			case 'nowrap':
			case 'readonly':
			case 'checked':
			case 'selected':
				if($attrib_value){
					$this->attribs[$attrib_name] = $attrib_name;
				}
				break;
			default:
				$this->attribs[$attrib_name] = $attrib_value;
		}
	}

	public function setStyle(string $type, string $val): void{
		$this->attribs['style'][trim($type)] = trim($val);
	}

	public function hasStyle(string $type): bool{
		return isset($this->attribs['style'][trim($type)]);
	}

	/**
	 * Function gets element attribute
	 *
	 * @param		$attrib_name							string
	 *
	 * @return		string
	 */
	public function getAttribute(string $attrib_name): string{
		return (string) ($this->attribs[$attrib_name] ?? '');
	}

	/**
	 * The function generate HTML code for the tag
	 *
	 * @param		we_html_baseElement $object
	 *
	 * @return		string
	 */
	public static function getHtmlCode(we_html_baseElement $object): string{
		return $object->getHTML();
	}

	/**
	 *
	 * @param array<string,string|array> $attr
	 * @return string
	 */
	public static function getAttribsAsString(array $attr): string{
		$out = '';
		foreach($attr as $k => $v){
			switch($k){
				case 'style':
					if($v){
						$out .= ' ' . $k . '="';
						foreach($v as $kk => $vv){
							$out .= $kk . ':' . $vv . ';';
						}
						$out .= '"';
					}
					break;
				case 'SCRIPTABLE'://FIXME: remove
				case 'MAYSCRIPT':
					$out .= ' ' . $k;
					break;
				default:
					if($v !== ''){
						if(is_array($v)){
							t_e($v);
						}
						$out .= ' ' . $k . '="' . $v . '"';
					} else {
						//no empty attribs
						//$out .= ' ' . $k . '=""';
					}
					break;
			}
		}
		return $out;
	}

	public function getHTML(): string{
		return '<' . $this->tag_name . self::getAttribsAsString($this->attribs) .
			($this->need_end_tag === self::SELF_CLOSE ? '/>' : '>' .
			$this->content .
			($this->need_end_tag ? '</' . $this->tag_name . '>' : '')
			);
	}

	public function getContent(): string{
		return $this->content;
	}

}
